﻿using ComPDFKitViewer;
using ComPDFKitViewer.PdfViewer;
using Microsoft.Win32;
using PDF_Office.Model;
using PDF_Office.Model.PropertyPanel.AnnotPanel;
using Prism.Commands;
using Prism.Mvvm;
using Prism.Regions;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;

namespace PDF_Office.ViewModels.PropertyPanel
{
    public class TextEditPropertyViewModel : BindableBase, INavigationAware
    {
        #region 属性

        #region 编辑PDF全局

        private bool _isTextEdit = true;
        public bool IsTextEdit
        {
            get { return _isTextEdit; }
            set
            {
                SetProperty(ref _isTextEdit, value);

            }
        }


        #endregion



        #region 文本属性

        private double _angle;
        public double Angle
        {
            get { return _angle; }
            set
            {
                SetProperty(ref _angle, value);

            }
        }

        private Brush selectColor = new SolidColorBrush(Colors.Black);
        public Brush SelectColor
        {
            get { return selectColor; }
            set
            {
                SetProperty(ref selectColor, value);

                if (TextEditEvent != null)
                {
                    bool isok = TextEditEvent.FontColor.A != (SelectColor as SolidColorBrush).Color.A ||
                         TextEditEvent.FontColor.B != (SelectColor as SolidColorBrush).Color.B ||
                         TextEditEvent.FontColor.G != (SelectColor as SolidColorBrush).Color.G ||
                         TextEditEvent.FontColor.R != (SelectColor as SolidColorBrush).Color.R;

                    if (isok)
                    {
                        TextEditEvent.FontColor = (SelectColor as SolidColorBrush).Color;
                        TextEditEvent.UpdatePDFEditByEventArgs();
                    }

                }


            }
        }

        private FontFamily fontFamily = new FontFamily("Courier");
        public FontFamily TextFontFamily
        {
            get { return fontFamily; }
            set
            {
                SetProperty(ref fontFamily, value);
                if (TextEditEvent != null)
                {
                    TextEditEvent.FontFamily = fontFamily;
                    TextEditEvent.UpdatePDFEditByEventArgs();
                }
            }
        }

        private FontWeight fontWeights = FontWeights.Normal;
        public FontWeight TextFontWeights
        {
            get { return fontWeights; }
            set
            {
                SetProperty(ref fontWeights, value);
                if (TextEditEvent != null)
                {
                    TextEditEvent.FontWeight = fontWeights;
                    TextEditEvent.UpdatePDFEditByEventArgs();
                }
            }
        }

        private FontStyle fontStyle = FontStyles.Normal;
        public FontStyle TextFontStyle
        {
            get { return fontStyle; }
            set
            {
                SetProperty(ref fontStyle, value);
                if (TextEditEvent != null)
                {
                    TextEditEvent.FontStyle = fontStyle;
                    TextEditEvent.UpdatePDFEditByEventArgs();
                }
            }
        }


        private int fontSize = 24;
        public int TextFontSize
        {
            get { return fontSize; }
            set
            {
                SetProperty(ref fontSize, value);
                if (TextEditEvent != null)
                {
                    TextEditEvent.FontSize = fontSize;
                    TextEditEvent.UpdatePDFEditByEventArgs();
                }
            }
        }

        private List<FontStyleItem> fontStyleList = new List<FontStyleItem>();
        public List<FontStyleItem> FontStyleList
        {
            get { return fontStyleList; }
            set
            {
                SetProperty(ref fontStyleList, value);
            }
        }
        #endregion

        #region 图像属性

        private bool _isCrop = false;
        public bool IsCrop
        {
            get { return _isCrop; }
            set
            {
                SetProperty(ref _isCrop, value);

            }
        }


        private System.Windows.Media.Imaging.BitmapSource _currentImg;
        public System.Windows.Media.Imaging.BitmapSource CurrentImg
        {
            get { return _currentImg; }
            set
            {
                SetProperty(ref _currentImg, value);
               
            }
        }

        #endregion

        #endregion


        #region Command

        #region 文本Command
        public DelegateCommand AddTextCommand { get; set; }
        public DelegateCommand AddImgCommand { get; set; }

        public DelegateCommand<object> SelectedColorCommand { get; set; }
        public DelegateCommand<object> SelectedFontStyleCommand { get; set; }
        public DelegateCommand<object> FontFamilyChangedCommand { get; set; }
        public DelegateCommand<object> FontStyleChangedCommand { get; set; }
        public DelegateCommand<object> FontSizeChangedCommand { get; set; }

        public DelegateCommand<object> TextAlignCheckedCommand { get; set; }

        #endregion

        #region 图像Command
        public DelegateCommand ReplaceImgCommand { get; set; }
        public DelegateCommand ExportImgCommand { get; set; }
        public DelegateCommand CropImgCommand { get; set; }
        public DelegateCommand ImgAlignCheckedCommand { get; set; }
        

        /// <summary>
        /// 逆时针旋转
        /// </summary>
        public DelegateCommand AntiClockwiseCommand { get; set; }
        /// <summary>
        /// 顺时针旋转
        /// </summary>
        public DelegateCommand ClockwiseCommand { get; set; }
        public DelegateCommand CropModeCommand { get; set; }
        public DelegateCommand CancelCropCommand { get; set; }

        #endregion

        #endregion

        private ComPDFKitViewer.PDFEditEvent TextEditEvent;
        public TextEditPropertyViewModel()
        {
            InitVariable();
            InitCommand();
        }

        private void InitVariable()
        {
            InitFontStyles();
        }

        private void InitFontStyles()
        {
            FontStyleList = LoadFontStyle.Load();
        }

        private void InitCommand()
        {
            AddTextCommand = new DelegateCommand(AddText);
            AddImgCommand = new DelegateCommand(AddImg);
            ReplaceImgCommand = new DelegateCommand(ReplaceImg);
            ExportImgCommand = new DelegateCommand(ExportImg);
            CropImgCommand = new DelegateCommand(CropImg);
            ImgAlignCheckedCommand = new DelegateCommand(ImgAlignChecked);

            SelectedColorCommand = new DelegateCommand<object>(SelectedColor);
            SelectedFontStyleCommand = new DelegateCommand<object>(SelectedFontStyle);
            FontFamilyChangedCommand = new DelegateCommand<object>(FontFamilyChanged);
            FontStyleChangedCommand = new DelegateCommand<object>(FontStyleChanged);
            FontSizeChangedCommand = new DelegateCommand<object>(FontSizeChanged);
            TextAlignCheckedCommand = new DelegateCommand<object>(TextAlignChecked);

            AntiClockwiseCommand = new DelegateCommand(AntiClockwise);
            ClockwiseCommand = new DelegateCommand(Clockwise);
            CropModeCommand = new DelegateCommand(CropMode);
            CancelCropCommand = new DelegateCommand(CancelCropImg);
        }

        private void ImgAlignChecked()
        {
           
        }

        private void CancelCropImg()
        {
            if (TextEditEvent != null)
            {
                TextEditEvent.ClipImage = false;
                TextEditEvent.CancelClip();
                
                TextEditEvent.UpdatePDFEditByEventArgs();
                IsCrop = false;
            }
        }

        private void CropImg()
        {
           if(TextEditEvent != null)
            {
                TextEditEvent.SaveClip();
                TextEditEvent.ClipImage = false;
                TextEditEvent.UpdatePDFEditByEventArgs();
                IsCrop = false;
            }
        }

        private void ExportImg()
        {
            if (PDFViewer == null || TextEditEvent == null || TextEditEvent.EditType != ComPDFKit.PDFPage.CPDFEditType.EditImage) return;

            //SaveFileDialog saveFileDialog = new SaveFileDialog();
            //saveFileDialog.Filter = "png|*.png;|Image|*.gif;*.jpg;*.jpeg;*.bmp;*.jfif;*.png;";
            //saveFileDialog.FileName = "编辑PDF导出图片";

            System.Windows.Forms.FolderBrowserDialog folder = new System.Windows.Forms.FolderBrowserDialog();
            folder.SelectedPath = Environment.GetFolderPath(Environment.SpecialFolder.UserProfile);

            if (folder.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                if (string.IsNullOrEmpty(folder.SelectedPath))
                    return;

                var keyValueList = PDFViewer.GetSelectedImages();
                int i = 0;
                foreach (var bitmap in keyValueList)
                {
                   foreach(var bitmapItem in bitmap.Value)
                    {
                        bitmapItem.Save(folder.SelectedPath + "\\" + i + ".png", System.Drawing.Imaging.ImageFormat.Png);
                        i++;
                    }
                }


                
            }
        }

        private void AddImg()
        {
            OpenFileDialog openFileDialog = new OpenFileDialog();
            openFileDialog.Filter = "png|*.png;|Image|*.gif;*.jpg;*.jpeg;*.bmp;*.jfif;*.png;";
            openFileDialog.Multiselect = true;
            if ((bool)openFileDialog.ShowDialog())
            {
                if (string.IsNullOrEmpty(openFileDialog.FileName) == false)
                {
                    PDFViewer.SetPDFEditCreateType(ComPDFKit.PDFPage.CPDFEditType.EditImage);
                    PDFViewer.AddPDFEditImage(openFileDialog.FileName);
                }
            }
        }

        private void ReplaceImg()
        {
            OpenFileDialog openFileDialog = new OpenFileDialog();
            openFileDialog.Filter = "png|*.png;|Image|*.gif;*.jpg;*.jpeg;*.bmp;*.jfif;*.png;";
            openFileDialog.Multiselect = true;
            if ((bool)openFileDialog.ShowDialog())
            {
                if (string.IsNullOrEmpty(openFileDialog.FileName) == false)
                {
                    PDFViewer.SetPDFEditCreateType(ComPDFKit.PDFPage.CPDFEditType.EditImage);
                    TextEditEvent.ReplaceImagePath = openFileDialog.FileName;
                    TextEditEvent.UpdatePDFEditByEventArgs();
                }
            }
        }

        private void AddText()
        {
            PDFViewer.SetPDFEditCreateType(ComPDFKit.PDFPage.CPDFEditType.EditText);
        }

        private void TextAlignChecked(object obj)
        {
           if((string)obj != null && TextEditEvent != null)
            {
                switch((string)obj)
                {
                    case "AlignLeft":
                        TextEditEvent.TextAlign = ComPDFKit.PDFPage.Edit.TextAlignType.AlignLeft;
                        break;

                    case "AlignCenter":
                        TextEditEvent.TextAlign = ComPDFKit.PDFPage.Edit.TextAlignType.AlignMiddle;
                        break;

                    case "AlignRight":
                        TextEditEvent.TextAlign = ComPDFKit.PDFPage.Edit.TextAlignType.AlignRight;
                        break;
                        
                    case "Align":
                        TextEditEvent.TextAlign = ComPDFKit.PDFPage.Edit.TextAlignType.AlignNone;
                        break;
                }
                TextEditEvent.UpdatePDFEditByEventArgs();
            }
        }

        private void Clockwise()
        {
            Angle = TextEditEvent.Rotate = TextEditEvent.Rotate + 90;
            TextEditEvent.UpdatePDFEditByEventArgs();
        }

        private void AntiClockwise()
        {
            Angle = TextEditEvent.Rotate = TextEditEvent.Rotate - 90;
            TextEditEvent.UpdatePDFEditByEventArgs();
        }


        private void CropMode()
        {
            IsCrop = true;
            if(TextEditEvent != null)
            {
                TextEditEvent.ClipImage = true;
                TextEditEvent.UpdatePDFEditByEventArgs();
            }
        }

        private void SelectedColor(object obj)
        {
            if (obj != null)
            {
                var colorValue = (Color)obj;
                if (colorValue != null)
                {

                    SelectColor = new SolidColorBrush(colorValue);
                }
            }
        }

        private void SelectedFontStyle(object obj)
        {
            if (obj != null && (FontStyleItem)obj != null)
            {
                var item = (FontStyleItem)obj;

            }
        }


        private void FontFamilyChanged(object obj)
        {
            if (obj != null)
            {
                if ((int)obj > -1)
                {
                    if ((int)obj == 0)
                    {

                        TextFontFamily = new FontFamily("Courier");
                    }


                    if ((int)obj == 1)
                    {
                        TextFontFamily = new FontFamily("Helvetica");

                    }


                    if ((int)obj == 2)
                    {
                        TextFontFamily = new FontFamily("Times");

                    }

                }
            }

        }



        private void FontStyleChanged(object obj)
        {
            if (obj != null)
            {
                var item = (ComboBoxItem)obj;
                var content = (string)item.Content;
                if (content != null)
                {
                    if (content == "Regular")
                    {
                        TextFontWeights = FontWeights.Normal;
                        TextFontStyle = FontStyles.Normal;
                    }


                    if (content == "Bold")
                    {
                        TextFontWeights = FontWeights.Bold;
                        TextFontStyle = FontStyles.Normal;
                    }


                    if (content == "Italic")
                    {
                        TextFontWeights = FontWeights.Normal;
                        TextFontStyle = FontStyles.Italic;
                    }

                    if (content == "Bold Italic")
                    {
                        TextFontWeights = FontWeights.Bold;
                        TextFontStyle = FontStyles.Italic;
                    }

                }
            }
        }


        private void FontSizeChanged(object obj)
        {
            if (obj != null)
            {
                var item = (ComboBoxItem)obj;
                var content = (string)item.Content;
                if (content != null)
                {
                    var intData = int.Parse(content);
                    TextFontSize = intData;
                }
            }
        }




        public void OnNavigatedTo(NavigationContext navigationContext)
        {

        }

        public bool IsNavigationTarget(NavigationContext navigationContext)
        {
            return true;
        }

        private CPDFViewer PDFViewer;
        public void OnNavigatedFrom(NavigationContext navigationContext)
        {
            navigationContext.Parameters.TryGetValue<CPDFViewer>(ParameterNames.PDFViewer, out PDFViewer);
            if(PDFViewer != null)
            {
                PDFViewer.PDFEditActiveHandler -= PDFViewer_PDFEditActiveHandler;
                PDFViewer.PDFEditActiveHandler += PDFViewer_PDFEditActiveHandler; 

                PDFViewer.PDFEditCommandHandler -= PDFViewer_PDFEditCommandHandler;
                PDFViewer.PDFEditCommandHandler += PDFViewer_PDFEditCommandHandler;
                
            }
        }

        private void PDFViewer_PDFEditCommandHandler(object sender, PDFEditCommand e)
        {
           
            ContextMenu popMenu = null;

            switch (e.CommandType)
            {
                case CommandType.Context:
                   
                    if (e.EditType == ComPDFKit.PDFPage.CPDFEditType.None)
                    {
                        popMenu = App.Current.FindResource("NoneMenu") as ContextMenu;
                        if(popMenu != null && popMenu.Items.Count == 4)
                        {
                            //复制
                            MenuItem menuItem = popMenu.Items[0] as MenuItem;
                            menuItem.CommandTarget = (UIElement)sender;
                            menuItem.Command = ApplicationCommands.Copy;
                            //添加文本
                            menuItem = popMenu.Items[1] as MenuItem;
                            menuItem.CommandTarget = (UIElement)sender;
                            menuItem.Command = AddTextCommand; 
                            //添加图像
                             menuItem = popMenu.Items[2] as MenuItem;
                            menuItem.CommandTarget = (UIElement)sender;
                            menuItem.Command = AddImgCommand;
                            //删除
                            menuItem = popMenu.Items[3] as MenuItem;
                            menuItem.CommandTarget = (UIElement)sender;
                            menuItem.Command = ApplicationCommands.Delete;

                            e.PopupMenu = popMenu;
                            if (e.PopupMenu != null)
                            {
                                e.Handle = true;
                            }
                        }
                    }
                    else if(e.EditType == ComPDFKit.PDFPage.CPDFEditType.EditImage)
                    {
                        popMenu = App.Current.FindResource("SelectImgMenu") as ContextMenu;
                        if (popMenu != null && popMenu.Items.Count == 8)
                        {
                            //复制
                            MenuItem menuItem = popMenu.Items[0] as MenuItem;
                            menuItem.CommandTarget = (UIElement)sender;
                            menuItem.Command = ApplicationCommands.Copy;
                            //剪切
                            menuItem = popMenu.Items[1] as MenuItem;
                            menuItem.CommandTarget = (UIElement)sender;
                            menuItem.Command = ApplicationCommands.Cut;
                            //粘贴
                            menuItem = popMenu.Items[2] as MenuItem;
                            menuItem.CommandTarget = (UIElement)sender;
                            menuItem.Command = ApplicationCommands.Paste;
                            //删除
                            menuItem = popMenu.Items[3] as MenuItem;
                            menuItem.CommandTarget = (UIElement)sender;
                            menuItem.Command = ApplicationCommands.Delete;
                            //裁剪
                            menuItem = popMenu.Items[4] as MenuItem;
                            menuItem.CommandTarget = (UIElement)sender;
                            menuItem.Command = CropModeCommand;
                            //替换
                            menuItem = popMenu.Items[5] as MenuItem;
                            menuItem.CommandTarget = (UIElement)sender;
                            menuItem.Command = ReplaceImgCommand;
                            //导出
                            menuItem = popMenu.Items[6] as MenuItem;
                            menuItem.CommandTarget = (UIElement)sender;
                            menuItem.Command = ExportImgCommand;
                            //排列
                            menuItem = popMenu.Items[7] as MenuItem;
                            menuItem.CommandTarget = (UIElement)sender;
                            menuItem.Command = ApplicationCommands.Cut;
                            e.PopupMenu = popMenu;
                            if (e.PopupMenu != null)
                            {
                                e.Handle = true;
                            }
                        }
                    }

            break;

                default:
                    e.DoCommand();
                    break;

            }
        }

        private void PDFViewer_PDFEditActiveHandler(object sender, List<PDFEditEvent> e)
        {
            if(e != null && e.Count > 0)
            {
                IsTextEdit = (e[0].EditType == ComPDFKit.PDFPage.CPDFEditType.EditText);
                TextEditEvent = e[0];
                
                if (IsTextEdit == false)
                {

                    var list = PDFViewer.GetSelectedImages();
                    if(list != null && list.Count > 0)
                    {
                        System.Drawing.Bitmap bitmap = null;
                        foreach (var item in list)
                        {
                            if(item.Value.Count>0)
                            {
                                bitmap = item.Value[0];
                                break;
                            }
                        }

                        if(bitmap != null)
                        {
                            IntPtr ip = bitmap.GetHbitmap();
                            System.Windows.Media.Imaging.BitmapSource bitmapSource = System.Windows.Interop.Imaging.CreateBitmapSourceFromHBitmap(ip, IntPtr.Zero, Int32Rect.Empty,
                            System.Windows.Media.Imaging.BitmapSizeOptions.FromEmptyOptions());
                            CurrentImg = bitmapSource;
                        }
                       
                    }
                    

                }

            }
            else
            {
                IsTextEdit = true;
            }
        }
    }
}
